import { middleLayerBase } from "./middleLayerBase";
import { initAir } from './air';

const { ccclass, property } = cc._decorator;

@ccclass
export default class NewClass extends middleLayerBase {
  showMask() {
    const mask = cc.find('middleLayer/mask');
    mask.active = true;
    cc.tween(mask)
      .set({ opacity: 0 })
      .to(0.5, { opacity: 0 })
      .start();
  }

  hideMask() {
    const mask = cc.find('middleLayer/mask');
    mask.stopAllActions();
    cc.tween(mask)
      .set({ opacity: 0 })
      .to(0.5, { opacity: 0 })
      .call(() => {
        mask.active = false;
      })
      .start();
  }

  onCourseWareLoaded() {
    // 课件加载完成的钩子
    const process = cc.find(`middleLayer/pageProgress`);
    const bar = cc.find(`bar`, process);
    const barWidth = process.width * 1 / this.courses.length;
    cc.tween(bar).to(0.5, { width: barWidth }).start();
    const page = cc.find(`page`, process);
    page.getComponent(cc.Label).string = `${this.courseIndex+1}/${this.courses.length}`;
  }


  // 需要预加载的数量
  preloadCount = null;
  // 统计已加载的数量
  loadedCount = null;

  role;

  // 统一释放异步资源的方法
  async onLoad() {
    this.preloadCount = 0;
    this.loadedCount = 0;
    initAir(this);
    this.reWriteAir();
    this.role = 'student';
    this.onMiddleLayerLoadingFinish();
  }

  async start() {
    this.node.zIndex = 9999;
    cc.game.addPersistRootNode(this.node);
    global.middleLayer = cc.find('middleLayer').getComponent('middleLayer');

    // 预加载
    await this.preloadAll();
    cc.find(`middleLayer/preload`).active = false;

    this.loadOnlineCourseWare(23751);
  }

  async preloadAll() {
    if ((<any>window).preloadBundleAndSourcesFlag) {
      // 只加载一次就行了
      return;
    }
    (<any>window).preloadBundleAndSourcesFlag = true;
    let platform = "web_desktop";
    if (cc.sys.os == cc.sys.OS_IOS) {
      platform = "ios";
    } else if (cc.sys.os == cc.sys.OS_ANDROID) {
      platform = "android";
    }
    const datastr:any = await this.asyncCallNetworkApiGet(`/api/syllabus/v1/allbundles`, { orgid: 512, platform });
    const data = JSON.parse(datastr);
    if (data.rows && data.rows.length > 0) {
      this.preloadCount += data.rows.length;
    }
    const datastr1:any = await this.asyncCallNetworkApiGet(`/api/syllabus/v1/allresources`, { orgid: 512 });
    const data1 = JSON.parse(datastr1);
    if (data1.rows && data1.rows.length > 0) {
      this.preloadCount += data1.rows.length;
    }
    console.log('data.rows = ' + JSON.stringify(data.rows));
    await this.preloadSceneAndSource(data, data1);
  }

  preloadSceneAndSource(scenes, sources) {
    const p1 = this.batchPreloadScene(scenes.rows);
    const p2 = this.preloadSource(sources.rows);
    return Promise.all([p1, p2]);
  }

  batchPreloadScene(scenes = []) {
    return new Promise((resolve) => {
      const expects = [
        // 'op_03_2',  
        // 'OP10_online', 
      ];
      const _this = this;
      let i = 0;
      const load = function () {
        const scene = scenes[i];
        if (expects.includes(scene.sceneName)) {
          _this.loadedCount++;
          _this.updateProcessBar();
          i++;
          if (i < scenes.length) {
            setTimeout(() => {
              load();
            }, 1);
          } else {
            resolve();
          }
          return;
        }
        cc.assetManager.loadBundle(scene.bondleUrl, { version: scene.version }, (err, bundle) => {
          console.log(`${scene.bondleUrl}---场景开始loadBundle----`);
          if (!bundle) {
            _this.loadedCount++;
            _this.updateProcessBar();
            i++;
            if (i < scenes.length) {
              setTimeout(() => {
                load();
              }, 1);
            } else {
              resolve();
            }
            return;
          }
          bundle.preloadScene(scene.sceneName, () => { }, () => {
            console.log(`${scene.sceneName}场景预加载结束`);
            _this.loadedCount++;
            _this.updateProcessBar();
            i++;
            if (i < scenes.length) {
              setTimeout(() => {
                load();
              }, 1);
            } else {
              resolve();
            }
          });
        });
      }
  
      if (scenes && scenes.length > 0) {
        load();
      } else {
        resolve();
      }
    });
  }

  preloadSource(array) {
    return new Promise((resolve) => {
      const rows = array || [];
      let loadFlag = 0;
      const batchLoad = () => {
        let tempArr = rows.splice(0, 10);
        let len = tempArr.length;
        for (let i = 0; i < tempArr.length; i++) {
          cc.assetManager.loadRemote(tempArr[i], (err, texture) => {
            if (err) {
              console.log(err)
            }
            this.loadedCount++;
            this.updateProcessBar();
            if (i === len - 1) {
              batchLoad();
            }
            loadFlag ++;
            if (loadFlag == rows.length) {
              resolve();
            }
          });
        }
      }
      batchLoad();
    });
  }

  updateProcessBar() {
    const label = cc.find("middleLayer/preload/progress/page").getComponent(cc.Label);
    label.string = `${Math.floor(this.loadedCount/this.preloadCount*100)}%`;
    
    // 课件加载完成的钩子
    const process = cc.find(`middleLayer/preload/progress`);
    const bar = cc.find(`bar`, process);
    const barWidth = process.width / this.preloadCount * this.loadedCount;
    bar.width = barWidth;
  }

  onHomeworkFinish(callBack, data = null) {
    this.goNextPage();

    const process = cc.find(`middleLayer/pageProgress`);
    const bar = cc.find(`bar`, process);
    let barWidth = process.width / this.courses.length * (this.courseIndex+1);
    barWidth = barWidth>process.width?process.width:barWidth;

    cc.tween(bar).to(0.5, { width: barWidth }).start();
    const page = cc.find(`page`, process);
    page.getComponent(cc.Label).string = `${this.courseIndex+1}/${this.courses.length}`;
  }

  localMode = false;

  asyncDelayLog(str) {
    console.log(str);
    return new Promise((resolve, reject) => {
      setTimeout(() => {
        resolve(null);
      }, 1);
    });
  }

  initListener() {
    
  }

  log(str) {
    console.log(str);
  }

  reWriteAir() {
    (<any>window).courseware.getData = (callback) => {
      let data = null;
      if (this.courseItem && this.courseItem.data) {
        data = JSON.parse(this.courseItem.data);
      }
      this.log("===成功调用getData===" + JSON.stringify(data));
      callback && callback(data);
    }
    (<any>window).courseware.nextPage = () => {
      this.goNextPage();
    }
    (<any>window).courseware.beforePage = () => {
      this.goPrePage();
      
    }
    (<any>window).courseware.sendAnswer = (answerObj) => {
      this.log("===成功调用sendAnswer===" + JSON.stringify(answerObj));
      this.callNetworkApiPost(`/api/oxford/courseware/v1/${this.courseItem.id}/saveanswer`, answerObj, (data) => {
        answerObj.callback && answerObj.callback(JSON.stringify(data.row));
      });
    }
    (<any>window).courseware.getAnswer = (queryObj, callback) => {
      this.log("===成功调用getAnswer===" + JSON.stringify(queryObj));
      this.callNetworkApiGet(`/api/oxford/courseware/v1/${this.courseItem.id}/getanswer`, queryObj, (datastr) => {
        const data = JSON.parse(datastr);
        callback && callback(JSON.stringify(data.rows));
      });
    }
  }

  callNativeFunction(param) {
    const paramStr = JSON.stringify(param);
    if (cc.sys.isNative && cc.sys.os == cc.sys.OS_IOS) {
      return jsb.reflection.callStaticMethod('CocosMng', 'cocosWithNativeProtocol:', paramStr);
    } else if (cc.sys.isNative && cc.sys.os == cc.sys.OS_ANDROID) {
      return jsb.reflection.callStaticMethod('com/iplayabc/cocos/AppActivity', 'cocosWithNativeProtocol', '(Ljava/lang/String;)Ljava/lang/String;', paramStr);
    } else if (cc.sys.isNative && cc.sys.os == cc.sys.OS_WINDOWS) {
      console.log('汪汪汪')
    } else {
      console.error('非源生环境11');
    }
  }

  isShowWaiting = false;
  showWaitingLetters() {

    if (this.isShowWaiting) {
      return;
    }
    this.isShowWaiting = true;

    const layout = cc.find('middleLayer/layout');
    layout.removeAllChildren();

    const canvas = cc.find("Canvas");
    const bg = layout.addComponent(cc.Graphics);
    bg.fillRect(-canvas.width / 2, -canvas.height / 2, canvas.width, canvas.height);


    const laodingLayer = cc.find('middleLayer/loadingLayer');
    laodingLayer.active = true;

    const script = cc.find('middleLayer/loadingLayer/progress').getComponent('progressScript');
    script.setProgress(0);

    const id = setInterval(() => {
      // console.log('changeObj.value: ', changeObj.value);
      script.setProgress(changeObj.value);
    }, 1);

    const changeObj = { value: 0 };
    cc.tween(changeObj)
      .to(5, { value: 0.9 })
      .call(() => {
        clearInterval(id);
      })
      .start(); 

  }

  hideWaitingLetters() {
    const layout = cc.find('middleLayer/layout');
    layout.active = false;
    const laodingLayer = cc.find('middleLayer/loadingLayer');
    laodingLayer.active = false;
  }

  getRainbowColorList() {
    return [
      cc.color(255, 255, 0),
      cc.color(128, 255, 0),
      cc.color(0, 255, 0),
      cc.color(0, 0, 0),

      cc.color(0, 255, 128),
      cc.color(0, 255, 255),

      cc.color(0, 128, 255),
      cc.color(0, 0, 255),
      cc.color(128, 0, 255),
      cc.color(255, 0, 255),
      cc.color(255, 0, 128),

      cc.color(255, 0, 0),
      cc.color(255, 128, 0),
      cc.color(255, 255, 0),
    ];
  }

  onMiddleLayerLoadingFinish() {
		if (cc.sys.isNative && cc.sys.os == cc.sys.OS_IOS) {
			jsb.reflection.callStaticMethod(
				"RootViewController",
				"getMiddleLayerInfoFinish:",
				""
			);
		}
		if (cc.sys.isNative && cc.sys.os == cc.sys.OS_ANDROID) {
			jsb.reflection.callStaticMethod(
				"com/iplayabc/cocos/AppActivity",
				"getMiddleLayerInfoFinish",
				"(Ljava/lang/String;)Ljava/lang/String;",
				""
			);
		}
	}
}
